#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "../ssewrapper.h"



/*
 * Warren Hunt : 2006-10-09
 * Definition of the names of the axes and their order
 *
 */
#define $axisList NO_AXIS, X_AXIS, Y_AXIS, Z_AXIS
#define $nextAxis NO_AXIS, Y_AXIS, Z_AXIS, X_AXIS
#define $prevAxis NO_AXIS, Z_AXIS, X_AXIS, Y_AXIS
#define $printAxis '!', 'X', 'Y', 'Z'

enum Axis { $axisList };

// shuffleing and printing operations
__forceinline Axis next( const Axis axis ) { const static Axis nextAxis[4] = { $nextAxis }; return nextAxis[axis]; }
__forceinline Axis prev( const Axis axis ) { const static Axis prevAxis[4] = { $prevAxis }; return prevAxis[axis]; }
__forceinline char printAxis( const Axis axis ) { const static char printAxis[4] = { $printAxis }; return printAxis[axis]; }

// other Axis operations
__forceinline unsigned __int32 indexAxis( const Axis axis ) { return axis - 1; }
__forceinline sseMask axisMask( const __m128 anything = sseZeros() ) { return shiftLeft128<sizeof(float)>(sseOnes(anything)); }
__forceinline sseMask noAxisMask( const __m128 anything = sseZeros() ) { return shiftRight128<3*sizeof(float)>(sseOnes(anything)); }
__forceinline __m128 _my_set3_ps( const float input_0, const float input_1, const float input_2 ) { return _mm_set_ps(input_0, input_1, input_2, 0.0f); } // ma__int32ains little endian order
template<unsigned __int8 index_0, unsigned __int8 index_1, unsigned __int8 index_2> __forceinline __m128 shuffle( const __m128 input ) { return shuffle<NO_AXIS, index_0, index_1, index_2>(input); }
template<typename Type> Type &noAxisCell( const __m128 &input ) { assert(sizeof(Type) == sizeof(float)); return reinterpret_cast(Type)input; }

// boolean operations
__forceinline bool and3( const sseMask &input ) { return (input.mask()&0xe) == 0xe; }
__forceinline bool or3 ( const sseMask &input ) { return (input.mask()&0xe) != 0x0; }



// foreach loop over Axis
#define foreach_Axis(name) for ( Axis (name) = X_AXIS ; (unsigned __int32)(name) <= (unsigned __int32)Z_AXIS ; (reinterpret_cast(unsigned __int32)(name))++ )



/*
 * Warren Hunt : 2006-10-09
 *
 */
template<typename Type> class AxisIndexedArray {
	Type data[3];

public:
	__forceinline AxisIndexedArray( void ) {}
	__forceinline AxisIndexedArray( const Type &input_X, const Type &input_Y, const Type &input_Z )
	{
		data[indexAxis(X_AXIS)] = input_X;
		data[indexAxis(Y_AXIS)] = input_Y;
		data[indexAxis(Z_AXIS)] = input_Z;
	}
	__forceinline AxisIndexedArray( const Type input[3] )
	{
		data[indexAxis(X_AXIS)] = input[0];
		data[indexAxis(Y_AXIS)] = input[1];
		data[indexAxis(Z_AXIS)] = input[2];
	}

	__forceinline Type &operator []( const Axis &axis ) const { assume(axis != NO_AXIS); return ((Type *)data)[indexAxis(axis)]; }
	__forceinline Type &operator []( const Axis &axis )       { assume(axis != NO_AXIS); return data[indexAxis(axis)]; }
};
