#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "sse3vector.h"


/*
 * Warren Hunt : 2006-10-09
 * A three dimensional point class implemented using sse
 *
 */
class sse3Point {
	sse3Vector data;

public:
	__forceinline sse3Point( const sse3Vector &input = sse3Vector() )
		: data(input) {}
	__forceinline sse3Point( const float input_0, const float input_1, const float input_2 )
		: data(input_0, input_1, input_2) {}

	__forceinline float  operator []( const Axis axis ) const { assume(axis != NO_AXIS); return data[axis]; }
	__forceinline float &operator []( const Axis axis )       { assume(axis != NO_AXIS); return data[axis]; }
	template<typename Type> __forceinline Type &cell( void ) const { return data.cell<Type>(); }
	template<typename Type> __forceinline Type &cell( void )       { return data.cell<Type>(); }

	__forceinline sse3Point operator +( const sse3Vector &input ) const { return data + input; }
	__forceinline sse3Point operator -( const sse3Vector &input ) const { return data - input; }
	__forceinline sse3Vector operator -( const sse3Point &input ) const { return data - input.data; }

	__forceinline sse3Point operator +=( const sse3Vector &input ) { return *this = *this + input; }
	__forceinline sse3Point operator -=( const sse3Vector &input ) { return *this = *this - input; }

	__forceinline sseMask operator ==( const sse3Point &input ) const { return data == input.data; }
	__forceinline sseMask operator !=( const sse3Point &input ) const { return data != input.data; }
	__forceinline sseMask operator < ( const sse3Point &input ) const { return data <  input.data; }
	__forceinline sseMask operator <=( const sse3Point &input ) const { return data <= input.data; }
	__forceinline sseMask operator > ( const sse3Point &input ) const { return data >  input.data; }
	__forceinline sseMask operator >=( const sse3Point &input ) const { return data >= input.data; }

	__forceinline sse3Vector vector( void ) const { return data; }

	friend __forceinline sse3Point min( const sse3Point &, const sse3Point & );
	friend __forceinline sse3Point max( const sse3Point &, const sse3Point & );
	friend __forceinline sse3Point blend( const sseMask &, const sse3Point &, const sse3Point & );
	friend __forceinline sse3Point &safeCopy( sse3Point &value, const sse3Point &input );
};

__forceinline sse3Point min( const sse3Point &input_0, const sse3Point &input_1 ) { return min(input_0.data, input_1.data); }
__forceinline sse3Point max( const sse3Point &input_0, const sse3Point &input_1 ) { return max(input_0.data, input_1.data); }

__forceinline sse3Point blend( const sseMask &mask, const sse3Point &input_0, const sse3Point &input_1 ) { return blend(mask, input_0.data, input_1.data); }
__forceinline sse3Point rotateAxesForward( const sse3Point &input ) { return rotateAxesForward(input.vector()); }
__forceinline sse3Point rotateAxesReverse( const sse3Point &input ) { return rotateAxesReverse(input.vector()); }
__forceinline sse4Floats expand( const sse3Point &input, const Axis axis ) { return expand(input.vector(), axis); }
__forceinline sse1Float extract( const sse3Point &input, const unsigned __int8 index ) { return extract(input.vector(), index); }

__forceinline sse3Point &safeCopy( sse3Point &value, const sse3Point &input ) { safeCopy(value.data, input.data); return value; }
