#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "../ssewrapper.h"
#include "axis.h"



/*
 * Warren Hunt : 2006-10-09
 * A three dimensional vector class implemented using sse
 *
 */

class sse3Vector {
	sse4Floats data;

public:
	__forceinline sse3Vector( const sse4Floats &input = sseZeros() )
		: data(input) {}
	__forceinline sse3Vector( const float input_0, const float input_1, const float input_2 )
		: data(_my_set3_ps(input_2, input_1, input_0)) {}

	__forceinline float  operator []( const Axis axis ) const { assume(axis != NO_AXIS); return data[axis]; }
	__forceinline float &operator []( const Axis axis )       { assume(axis != NO_AXIS); return data[axis]; }
	template<typename Type> __forceinline Type &cell( void ) const { return noAxisCell<Type>(data.data); }
	template<typename Type> __forceinline Type &cell( void )       { return noAxisCell<Type>(data.data); }

	__forceinline sse3Vector operator +( const sse3Vector &input ) const { return data + input.data; }
	__forceinline sse3Vector operator -( const sse3Vector &input ) const { return data - input.data; }
	__forceinline sse3Vector operator *( const sse1Float &input ) const { return data * expand(input); }
	__forceinline sse3Vector operator /( const sse1Float &input ) const { return data / expand(input); }

	__forceinline sse3Vector &operator +=( const sse3Vector &input ) { return *this = *this + input; }
	__forceinline sse3Vector &operator -=( const sse3Vector &input ) { return *this = *this - input; }
	__forceinline sse3Vector &operator *=( const sse1Float &input ) { return *this = *this * input; }
	__forceinline sse3Vector &operator /=( const sse1Float &input ) { return *this = *this / input; }

	__forceinline sse1Float operator *( const sse3Vector &input ) const { return hadd((*this)(input)); } // dot product
	__forceinline sse3Vector operator ^( const sse3Vector &input ) const { return rotateAxesForward(*this).data*rotateAxesReverse(input).data - rotateAxesForward(input).data*rotateAxesReverse(*this).data; } // cross product
	__forceinline sse3Vector operator ()( const sse3Vector &input ) const { return data*input.data; } // (a)(b) for scaler multiply of a and b

	__forceinline sseMask operator ==( const sse3Vector &input ) const { return data == input.data; }
	__forceinline sseMask operator !=( const sse3Vector &input ) const { return data != input.data; }
	__forceinline sseMask operator < ( const sse3Vector &input ) const { return data <  input.data; }
	__forceinline sseMask operator <=( const sse3Vector &input ) const { return data <= input.data; }
	__forceinline sseMask operator > ( const sse3Vector &input ) const { return data >  input.data; }
	__forceinline sseMask operator >=( const sse3Vector &input ) const { return data >= input.data; }

	__forceinline sse3Vector operator -( void ) const { return -data; }
	__forceinline sse3Vector operator ~( void ) const { return ~data; }

	__forceinline sse1Float lengthSquared( void ) const { return *this * *this; }
	__forceinline sse1Float length( void ) const { return sqrt1(lengthSquared()); }
	__forceinline sse3Vector direction( void ) const { return *this * rcpsqrt1(lengthSquared()); }

	__forceinline void print( void ) const { printf("(%f, %f, %f)", data[X_AXIS], data[Y_AXIS], data[Z_AXIS]); }

	friend __forceinline sse3Vector min( const sse3Vector &, const sse3Vector & );
	friend __forceinline sse3Vector max( const sse3Vector &, const sse3Vector & );
	friend __forceinline sse3Vector blend( const sseMask &, const sse3Vector &, const sse3Vector & );
	friend __forceinline sse3Vector rotateAxesForward( const sse3Vector & );
	friend __forceinline sse3Vector rotateAxesReverse( const sse3Vector & );
	friend __forceinline sse4Floats expand( const sse3Vector &input, const Axis axis );
	friend __forceinline sse1Float extract( const sse3Vector &input, const unsigned __int8 index );
	friend __forceinline sse1Float  hadd( const sse3Vector & );
	friend __forceinline sse3Vector &safeCopy( sse3Vector &value, const sse3Vector &input );
};

__forceinline sse3Vector operator *( const sse1Float &input, const sse3Vector &value ) { return value * input; }

__forceinline sse3Vector min( const sse3Vector &input_0, const sse3Vector &input_1 ) { return min4(input_0.data, input_1.data); }
__forceinline sse3Vector max( const sse3Vector &input_0, const sse3Vector &input_1 ) { return max4(input_0.data, input_1.data); }
__forceinline sse3Vector clamp4( const sse3Vector &input, const sse3Vector &lowerBound, const sse3Vector &upperBound ) { return min(max(input, lowerBound), upperBound); }

__forceinline sse3Vector blend( const sseMask &mask, const sse3Vector &input_0, const sse3Vector &input_1 ) { return blend(mask, input_0.data, input_1.data); }
__forceinline sse3Vector rotateAxesForward( const sse3Vector &input ) { return shuffle<$nextAxis>(input.data); }
__forceinline sse3Vector rotateAxesReverse( const sse3Vector &input ) { return shuffle<$prevAxis>(input.data); }
__forceinline sse4Floats expand( const sse3Vector &input, const Axis axis ) { return expand(input.data, (unsigned __int8)axis); }
__forceinline sse1Float extract( const sse3Vector &input, const unsigned __int8 index ) { return extract(input.data, index); }
__forceinline sse1Float hadd( const sse3Vector &input ) { return (expand(input, X_AXIS) + expand(input, Y_AXIS) + expand(input, Z_AXIS)).data; }

__forceinline sse3Vector &safeCopy( sse3Vector &value, const sse3Vector &input ) { return value = blend(axisMask(value.data.data), input, value); }
