#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "sse3point.h"



/*
 * Warren Hunt : 2006-10-09
 * Axis aligned bounding box class implemented in SSE
 *
 */
struct sseAABBox {
	sse3Point lo;
	sse3Point hi;

	__forceinline sseAABBox( void )
		: lo(), hi() {}
	__forceinline sseAABBox( const sse3Point &input_lo, const sse3Point &input_hi )
		: lo(input_lo), hi(input_hi) {}

	__forceinline bool isEmpty( void ) const { return or3(lo >= hi); }

	__forceinline void makeNonPlanar( void )
	{
		sseMask mask(~(lo != hi)); // not the same as == for denormals
		lo -= expand(0.001f)&mask;
		hi += expand(0.001f)&mask;
	}

	__forceinline sseAABBox boundLower( const Axis axis, const float plane ) const { sseAABBox rvalue(*this); rvalue.lo[axis] = plane; return rvalue; }
	__forceinline sseAABBox boundUpper( const Axis axis, const float plane ) const { sseAABBox rvalue(*this); rvalue.hi[axis] = plane; return rvalue; }

	__forceinline sse1Float surfaceAreaHalf( void ) const { sse3Vector length(hi - lo); return length*rotateAxesForward(length); }
	__forceinline sse1Float surfaceArea( void ) const { return surfaceAreaHalf()*2.0f; }
	__forceinline sse3Vector areas( void ) const { sse3Vector length(hi - lo); return (rotateAxesForward(length))(rotateAxesReverse(length)); } // areas of the faces orthagonal to an Axis

	__forceinline Axis longestAxis( void ) const { const sse3Vector length = hi - lo; Axis longest = X_AXIS; if ( length[Y_AXIS] > length[longest] ) longest = Y_AXIS; if ( length[Z_AXIS] > length[longest] ) longest = Z_AXIS; return longest; }
	__forceinline Axis shortestAxis( void ) const { const sse3Vector length = hi - lo; Axis shortest = X_AXIS; if ( length[Y_AXIS] < length[shortest] ) shortest = Y_AXIS; if ( length[Z_AXIS] < length[shortest] ) shortest = Z_AXIS; return shortest; }

	__forceinline void print( void ) const { printf("(%9f -- %9f), (%9f -- %9f), (%9f -- %9f)\n", lo[X_AXIS], hi[X_AXIS], lo[Y_AXIS], hi[Y_AXIS], lo[Z_AXIS], hi[Z_AXIS]); }
};

__forceinline bool intersects( const sseAABBox &input_0, const sseAABBox &input_1 ) { return and3(max(input_0.lo, input_1.lo) < min(input_0.hi, input_1.hi)); }

__forceinline sseAABBox aabbUnion( const sse3Point &input_0, const sse3Point &input_1 ) { return sseAABBox(min(input_0, input_1), max(input_0, input_1)); }
__forceinline sseAABBox aabbUnion( const sseAABBox &input_0, const sseAABBox &input_1 ) { return sseAABBox(min(input_0.lo, input_1.lo), max(input_0.hi, input_1.hi)); }
__forceinline sseAABBox aabbUnion( const sseAABBox &input_0, const sse3Point &input_1 ) { return sseAABBox(min(input_0.lo, input_1), max(input_0.hi, input_1)); }
__forceinline sseAABBox aabbUnion( const sse3Point &input_0, const sseAABBox &input_1 ) { return sseAABBox(min(input_0, input_1.lo), max(input_0, input_1.hi)); }
