#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "samplestructs.h"
#include "kdframe.h"



/*
 * Warren Hunt : 2006-10-09
 * Chooses the best split on the bounds of a given AABBox
 *
 */
 __forceinline Split chooseSplitBounds( const KDTreeFrame &frame, const sseAABBox &objectBounds )
{
	const Sample3CostEvaluationCache costCache(frame.voxelBounds);
	return (Split3(Sample3(objectBounds.lo, sse3Vector(), expand(frame.totalCost)), costCache)
		<=  Split3(Sample3(objectBounds.hi, expand(frame.totalCost), sse3Vector()), costCache)).best();
}



/*
 * Warren Hunt : 2006-10-09
 * Choose the best split on the provided bounds on the provided axis
 *
 */
__forceinline Split chooseSplitBounds( const KDTreeFrame &frame, const Axis axis, const Sample4CostEvaluationCache &costCache, const BoundsPair &bound )
{
	return Split(Sample(bound.lo, 0.0f, frame.totalCost), axis, costCache, true)
		<= Split(Sample(bound.hi, frame.totalCost, 0.0f), axis, costCache, true);
}



/*
 * Warren Hunt : 2006-10-09
 * Chooses the best split on the object bounds on the provided axis 
 *
 */
__forceinline Split chooseSplitBounds( const KDTreeFrame &frame, const Axis axis, const Sample4CostEvaluationCache &costCache )
{
	assume(axis != NO_AXIS);
	const unsigned __int32 end = frame.range.end;

	// find the bounds and call the function that chooese over them
	sse1Float bounds_lower(frame(axis, frame.range.start).lo);
	sse1Float bounds_upper(frame(axis, frame.range.start).hi);
	for ( unsigned __int32 cursor = frame.range.start + 1 ; cursor < end ; cursor++ )
	{
		bounds_lower = min1(bounds_lower, sse1Float(frame(axis, cursor).lo));
		bounds_upper = max1(bounds_upper, sse1Float(frame(axis, cursor).hi));
	}
	return chooseSplitBounds(frame, axis, costCache, BoundsPair(bounds_lower[0], bounds_upper[0]));
}



/*
 * Warren Hunt : 2006-10-09
 * Chooses the best split on the object bounds across all axes
 *
 */
__forceinline Split chooseSplitBounds( const KDTreeFrame &frame )
{
	Split bestSplit = frame.defaultSplit();
	foreach_Axis(axis)
	{
		Sample4CostEvaluationCache costCache(frame.voxelBounds, axis);
		bestSplit <= chooseSplitBounds(frame, axis, costCache);
	}
	return bestSplit;
}
