#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "objectlist.h"



/*
 * Warren Hunt : 2006-10-09
 * Enumeration that defines the ways we can determine the "best" split
 *
 */
enum SAHType { SCANALL_ONE, SCANALL_ALL, SCANSAMPLE_ONE, SCANSAMPLE_ALL, SORT_ONE, SORT_ALL };

/*
 * Warren Hunt : 2006-10-09
 * Structure containing information related to the quality of a kd-tree split
 *
 */
struct KDTreeQuality {
	const unsigned __int32 smallCutoff;
	const SAHType small;
	const unsigned __int32 largeCutoff;
	const SAHType large;
	const float leafBonus;
	const float traversalStepCost;

	__forceinline KDTreeQuality( const unsigned __int32 input_smallCutoff, const SAHType input_small, const unsigned __int32 input_largeCutoff, const SAHType input_large, const float input_leafBonus, const float input_traversalStepCost )
		: smallCutoff(input_smallCutoff), small(input_small), largeCutoff(input_largeCutoff), large(input_large), leafBonus(input_leafBonus), traversalStepCost(input_traversalStepCost)
	{
		if ( traversalStepCost <= 0.0f )
		{
			printf("error: traversalStepCost too small!\n");
			exit(1);
		}
		if ( leafBonus > 1.0f || leafBonus < 0.0f )
		{
			printf("error: leafBonus out of range!, should be in (0,1]\n");
			exit(1);
		}
	}
};

/*
 * Warren Hunt : 2006-10-09
 * The default kd-tree split qulaities
 *
 */
const static KDTreeQuality KDTREEQUALITY_1SCAN		(0,		SCANALL_ONE,	36,		SCANSAMPLE_ONE,	0.85f,	1.0f);
const static KDTreeQuality KDTREEQUALITY_1SORT		(0,		SCANALL_ONE,	36,		SORT_ONE,		0.85f,	1.0f);
const static KDTreeQuality KDTREEQUALITY_3SCAN		(0,		SCANSAMPLE_ALL,	36,		SCANSAMPLE_ALL, 0.85f,	1.0f);
const static KDTreeQuality KDTREEQUALITY_3SORT		(0,		SCANALL_ALL,	36,		SORT_ALL,		0.85f,	1.0f);

const static KDTreeQuality KDTREEQUALITY_LOOSEBINS	(0,		SCANALL_ONE,	26,		SCANSAMPLE_ONE,	0.75f,	130.0f);
const static KDTreeQuality KDTREEQUALITY_BINS		(0,		SCANALL_ONE,	26,		SCANSAMPLE_ONE,	0.80f,	30.0f);
const static KDTreeQuality KDTREEQUALITY_VERYFAST	(0,		SCANALL_ONE,	26,		SCANSAMPLE_ONE,	0.85f,	2.0f);
const static KDTreeQuality KDTREEQUALITY_FAST		(16,	SCANALL_ONE,	36,		SCANSAMPLE_ONE,	0.85f,	1.0f);
const static KDTreeQuality KDTREEQUALITY_FAIR		(36,	SCANSAMPLE_ALL,	128,	SCANSAMPLE_ONE,	0.85f,	0.8f);
const static KDTreeQuality KDTREEQUALITY_AVERAGE	(36,	SCANSAMPLE_ALL,	256,	SCANSAMPLE_ONE,	0.875f,	0.5f);
const static KDTreeQuality KDTREEQUALITY_GOOD		(36,	SCANSAMPLE_ALL,	512,	SCANSAMPLE_ONE,	0.90f,	0.2f);
const static KDTreeQuality KDTREEQUALITY_BETTER		(36,	SCANSAMPLE_ALL,	1024,	SCANSAMPLE_ONE,	1.0f,	0.1f);
const static KDTreeQuality KDTREEQUALITY_BEST		(36,	SCANALL_ALL,	36,		SORT_ALL,		1.0f,	0.01f);

// an array containing the above
const static KDTreeQuality KDTREEQUALITY[10] = {
	KDTREEQUALITY_LOOSEBINS,
	KDTREEQUALITY_LOOSEBINS,
	KDTREEQUALITY_BINS,
	KDTREEQUALITY_VERYFAST,
	KDTREEQUALITY_FAST,
	KDTREEQUALITY_FAIR,
	KDTREEQUALITY_AVERAGE,
	KDTREEQUALITY_GOOD,
	KDTREEQUALITY_BETTER,
	KDTREEQUALITY_BEST
};



/*
 * Warren Hunt : 2006-10-09
 * The kd-tree structure contians a kd-tree and information about it
 *
 */
class KDTree {
	class KDTreeNode;

	// local variables
	Array<KDTreeNode> nodes;
	const sseAABBox worldBounds;
	class ReferenceArena &arena;
	unsigned __int32 top;
	KDTreeQuality treeQuality;

	// building functions
	void buildKDTreeNode( class KDTreeFrame &, const unsigned __int32 );
	unsigned __int32 getNextFree( void );

public:
	KDTree( const ObjectList &, const KDTreeQuality & );
	~KDTree( void );

	float cost( void ) const;

	friend class KDTreeFrame;
};
