#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "../basicgeometry.h"



/*
 * Warren Hunt : 2006-10-09
 * Structure containing a pair of values defining a segment along an axis
 *
 */
struct BoundsPair {
	float lo;
	float hi;

	__forceinline BoundsPair( void ) {}
	__forceinline BoundsPair( const float input_lo, const float input_hi )
		: lo(input_lo), hi(input_hi) { assume(input_lo <= input_hi); }
};

/*
 * Warren Hunt : 2006-10-09
 * Structure that contains list of all geometry relivant to a kd-tree build
 * Information is stored as and array of boundspairs along each axis
 *
 */
class ObjectList {
	AxisIndexedArray<Array<BoundsPair> > aabbData;
	Array<float> costData;
	Array<unsigned __int32> objectReferences;

public:
	const sseAABBox worldBounds;
	const unsigned __int32 size;
	const float totalCost;

	/*
	 * Warren Hunt : 2006-10-09
	 * Basic constructor reads from an AABBox array file dump
	 *
	 */
	__forceinline ObjectList( const char *fileName )
		: size(0), totalCost(0.0f)
	{
		FILE *inFile = fopen(fileName, "rb");
		if ( !inFile )
		{
			printf("could not open file: %s\n", fileName);
			exit(1);
		}
		fseek(inFile, 0, SEEK_END);
		reinterpret_cast(unsigned __int32)size = (unsigned __int32)(ftell(inFile)/sizeof(sseAABBox));
		rewind(inFile);
		Array<sseAABBox> objects(size);
		fread(&objects[0], sizeof(sseAABBox), size, inFile);
		fclose(inFile);


		foreach_Axis(axis)
			new (&aabbData[axis]) Array<BoundsPair>(size);
		new (&objectReferences) Array<unsigned __int32>(size);
		new (&costData) Array<float>(size);


		reinterpret_cast(sseAABBox)worldBounds = objects[0];
		for ( unsigned __int32 i = 0 ; i < size ; i++ )
		{
			objects[i].makeNonPlanar();
			foreach_Axis(axis)
			{
				aabbData[axis][i].lo = objects[i].lo[axis];
				aabbData[axis][i].hi = objects[i].hi[axis];
			}
			objectReferences[i] = i;
			costData[i] = 1.0f;
			reinterpret_cast(float)totalCost += costData[i];
			reinterpret_cast(sseAABBox)worldBounds = aabbUnion(worldBounds, objects[i]);
		}
	}

	// accessor functions
	__forceinline Array<BoundsPair> &operator []( const Axis axis ) const { return aabbData[axis]; }
	__forceinline float cost( const unsigned __int32 index ) const { assume(index < size); return costData[index]; }
	__inline sseAABBox aabb( const unsigned __int32 index ) const
	{
		assume(index < size);
		return sseAABBox(sse3Point(aabbData[X_AXIS][index].lo,
			                       aabbData[Y_AXIS][index].lo,
								   aabbData[Z_AXIS][index].lo),
						 sse3Point(aabbData[X_AXIS][index].hi,
								   aabbData[Y_AXIS][index].hi,
								   aabbData[Z_AXIS][index].hi));
	}
};