#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "referencearena.h"
#include "bounds.h"



/*
 * Warren Hunt : 2006-10-09
 * The inner scanning loop, scans 8 samples
 * Returns a bound on the given axis (this is all dead-code eliminated if not used)
 *
 */
__forceinline BoundsPair scan( Sample8 &samples, const KDTreeFrame &frame, const Axis axis )
{
	assume(axis != NO_AXIS);

	sse4Floats lower4_countBelow;
	sse4Floats upper4_countBelow;
	sse4Floats lower4_countAbove;
	sse4Floats upper4_countAbove;
	sse1Float bound_lo = frame(axis, frame.range.start).lo;
	sse1Float bound_hi = frame(axis, frame.range.start).hi;
	const Range localRange(frame.range);
	forRange(localRange, index)
	{
		const sse4Floats object_cost(expand(frame.cost(index)));
		const sse4Floats bounds_lo = expand(frame(axis, index).lo);
		const sse4Floats bounds_hi = expand(frame(axis, index).hi);
		bound_lo = min1(bound_lo, bounds_lo.data);
		bound_hi = max1(bound_hi, bounds_hi.data);
		lower4_countBelow += object_cost&(bounds_lo < samples.lower4.plane);
		upper4_countBelow += object_cost&(bounds_lo < samples.upper4.plane);
		lower4_countAbove += object_cost&(bounds_hi > samples.lower4.plane);
		upper4_countAbove += object_cost&(bounds_hi > samples.upper4.plane);
	}
	samples.lower4.countBelow = lower4_countBelow;
	samples.upper4.countBelow = upper4_countBelow;
	samples.lower4.countAbove = lower4_countAbove;
	samples.upper4.countAbove = upper4_countAbove;
	return BoundsPair(bound_lo[0], bound_hi[0]);
}



/*
 * Warren Hunt : 2006-10-09
 * Uses the adaptive scanning technique to attempt the best split across a given axis
 *
 */
Split chooseSplitScanSample( const KDTreeFrame &frame, const Axis axis )
{
	assume(axis != NO_AXIS);
	if ( veryclose(frame.voxelBounds.lo[axis], frame.voxelBounds.hi[axis]) )
		return frame.defaultSplit();
	sse1Float bounds_lower, bounds_upper;
	const Sample4CostEvaluationCache costCache(frame.voxelBounds, axis);

	// comb scan
	Sample8 firstSamples(frame.voxelBounds, axis);
	const BoundsPair bound = scan(firstSamples, frame, axis);

	// adaptive scan
	Sample8 secondSamples(frame.voxelBounds, axis, firstSamples, costCache);
	scan(secondSamples, frame, axis);

	return projectBest(firstSamples, secondSamples, costCache, axis)
		<= chooseSplitBounds(frame, axis, costCache, bound);
}



/*
 * Warren Hunt : 2006-10-09
 * Uses the adaptive scanning technique to attemp the best split across all axes
 *
 */
__forceinline Split chooseSplitScanSample( const KDTreeFrame &frame )
{
	Split bestSplit = frame.defaultSplit();
	foreach_Axis(axis)
		bestSplit <= chooseSplitScanSample(frame, axis);
	return bestSplit;
}
