#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "ssebasic.h"
#include "ssemask.h"



/*
 * Warren Hunt : 2006-10-09
 * A wrapper for a single float using sse
 * The other 3 values in the sse word are assumed to be undefined
 *
 */
struct sse1Float {
	__m128 data;
	__forceinline sse1Float( const __m128 input = sseZeros() )
		: data(input) {}
	__forceinline sse1Float( const float input )
		: data(_mm_set_ss(input)) {}

	__forceinline float  operator []( const unsigned __int32 index ) const { assume(index == 0); return ((float *)&data)[index]; }
	__forceinline float &operator []( const unsigned __int32 index )       { assume(index == 0); return ((float *)&data)[index]; }

	__forceinline sse1Float operator +( const sse1Float &input ) const { return _mm_add_ss(data, input.data); }
	__forceinline sse1Float operator -( const sse1Float &input ) const { return _mm_sub_ss(data, input.data); }
	__forceinline sse1Float operator *( const sse1Float &input ) const { return _mm_mul_ss(data, input.data); }
	__forceinline sse1Float operator /( const sse1Float &input ) const { return _mm_div_ss(data, input.data); }
	__forceinline sse1Float operator ()( const sse1Float &input ) const { return *this * input; }

	__forceinline sse1Float &operator +=( const sse1Float &input ) { return *this = *this + input; }
	__forceinline sse1Float &operator -=( const sse1Float &input ) { return *this = *this - input; }
	__forceinline sse1Float &operator *=( const sse1Float &input ) { return *this = *this * input; }
	__forceinline sse1Float &operator /=( const sse1Float &input ) { return *this = *this / input; }

	__forceinline bool operator ==( const sse1Float &input ) const { return (_mm_movemask_ps(_mm_cmpeq_ss (data, input.data))&0x1) != 0x0; }
	__forceinline bool operator !=( const sse1Float &input ) const { return (_mm_movemask_ps(_mm_cmpneq_ss(data, input.data))&0x1) != 0x0; }
	__forceinline bool operator < ( const sse1Float &input ) const { return (_mm_movemask_ps(_mm_cmplt_ss (data, input.data))&0x1) != 0x0; }
	__forceinline bool operator <=( const sse1Float &input ) const { return (_mm_movemask_ps(_mm_cmple_ss (data, input.data))&0x1) != 0x0; }
	__forceinline bool operator > ( const sse1Float &input ) const { return (_mm_movemask_ps(_mm_cmpgt_ss (data, input.data))&0x1) != 0x0; }
	__forceinline bool operator >=( const sse1Float &input ) const { return (_mm_movemask_ps(_mm_cmpge_ss (data, input.data))&0x1) != 0x0; }

	__forceinline sse1Float operator -( void ) const { return sse1Float() - *this; }
	__forceinline sse1Float operator ~( void ) const { return sse1Float(convert(cast(sse0x1(data)))) / *this; } //_mm_rcp_ss(data); }

	__forceinline sse1Float  operator & ( const sseMask &input ) const { return _mm_and_ps(input.data, data); }
	__forceinline sse1Float &operator &=( const sseMask &input )       { return *this = *this & input; }
};

__forceinline sse1Float operator &( const sseMask &input, const sse1Float &value ) { return value & input; }

__forceinline sse1Float sqrt1( const sse1Float &input ) { return _mm_sqrt_ss(input.data); }
__forceinline sse1Float rcpsqrt1( const sse1Float &input ) { return _mm_rsqrt_ss(input.data); }

__forceinline sse1Float min1( const sse1Float &input_0, const sse1Float &input_1 ) { return _mm_min_ss(input_0.data, input_1.data); }
__forceinline sse1Float max1( const sse1Float &input_0, const sse1Float &input_1 ) { return _mm_max_ss(input_0.data, input_1.data); }

__forceinline sse1Float blend( const sseMask &mask, const sse1Float &input_0, const sse1Float &input_1 ) { return blend(mask.data, input_0.data, input_1.data); }
 
// safeCopy preserves the origional values of the 3 undefined word entries
__forceinline sse1Float &safeCopy( sse1Float &value, const sse1Float &input ) { return value = blend(shiftRight128<3*sizeof(float)>(sseOnes(value.data)), input, value); }
