#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "ssebasic.h"
#include "ssemask.h"



/*
 * Warren Hunt : 2006-10-09
 * A wrapper for a 4-vector of floats using sse
 *
 */
struct sse4Floats {
	__m128 data;
	__forceinline sse4Floats( const __m128 input = sseZeros() )
		: data(input) {}
	__forceinline sse4Floats( const float input_0, const float input_1, const float input_2, const float input_3 )
		: data(_mm_set_ps(input_3, input_2, input_1, input_0)) {}

	__forceinline float  operator []( const unsigned __int32 index ) const { assume(index < 4); return ((float *)&data)[index]; }
	__forceinline float &operator []( const unsigned __int32 index )       { assume(index < 4); return ((float *)&data)[index]; }

	__forceinline sse4Floats operator +( const sse4Floats &input ) const { return _mm_add_ps(data, input.data); }
	__forceinline sse4Floats operator -( const sse4Floats &input ) const { return _mm_sub_ps(data, input.data); }
	__forceinline sse4Floats operator *( const sse4Floats &input ) const { return _mm_mul_ps(data, input.data); }
	__forceinline sse4Floats operator /( const sse4Floats &input ) const { return _mm_div_ps(data, input.data); }
	__forceinline sse4Floats operator ()( const sse4Floats &input ) const { return *this * input; }

	__forceinline sse4Floats &operator +=( const sse4Floats &input ) { return *this = *this + input; }
	__forceinline sse4Floats &operator -=( const sse4Floats &input ) { return *this = *this - input; }
	__forceinline sse4Floats &operator *=( const sse4Floats &input ) { return *this = *this * input; }
	__forceinline sse4Floats &operator /=( const sse4Floats &input ) { return *this = *this / input; }

	__forceinline sseMask operator ==( const sse4Floats &input ) const { return _mm_cmpeq_ps (data, input.data); }
	__forceinline sseMask operator !=( const sse4Floats &input ) const { return _mm_cmpneq_ps(data, input.data); }
	__forceinline sseMask operator < ( const sse4Floats &input ) const { return _mm_cmplt_ps (data, input.data); }
	__forceinline sseMask operator <=( const sse4Floats &input ) const { return _mm_cmple_ps (data, input.data); }
	__forceinline sseMask operator > ( const sse4Floats &input ) const { return _mm_cmpgt_ps (data, input.data); }
	__forceinline sseMask operator >=( const sse4Floats &input ) const { return _mm_cmpge_ps (data, input.data); }

	__forceinline sse4Floats operator -( void ) const { return sse4Floats() - *this; }
	__forceinline sse4Floats operator ~( void ) const { return sse4Floats(convert(cast(sse0x1(data)))) / *this; } //_mm_rcp_ps(data); }

	__forceinline sse4Floats  operator & ( const sseMask &input ) const { return _mm_and_ps(input.data, data); }
	__forceinline sse4Floats &operator &=( const sseMask &input )       { return *this = *this & input; }

	__forceinline void print( void ) const { printf("%f, %f, %f, %f\n", (*this)[0], (*this)[1], (*this)[2], (*this)[3]); }
};

__forceinline sse4Floats operator &( const sseMask &input, const sse4Floats &value ) { return value & input; }

__forceinline sse4Floats sqrt4( const sse4Floats &input ) { return _mm_sqrt_ps(input.data); }
__forceinline sse4Floats rcpsqrt4( const sse4Floats &input ) { return _mm_rsqrt_ps(input.data); }
__forceinline sse4Floats abs( const sse4Floats &input ) { return abs(input.data); }

__forceinline sse4Floats min4( const sse4Floats &input_0, const sse4Floats &input_1 ) { return _mm_min_ps(input_0.data, input_1.data); }
__forceinline sse4Floats max4( const sse4Floats &input_0, const sse4Floats &input_1 ) { return _mm_max_ps(input_0.data, input_1.data); }
__forceinline sse4Floats clamp4( const sse4Floats &input, const sse4Floats &lowerBound, const sse4Floats &upperBound ) { return min4(max4(input, lowerBound), upperBound); }

__forceinline sse4Floats blend( const sseMask &mask, const sse4Floats &input_0, const sse4Floats &input_1 ) { return blend(mask.data, input_0.data, input_1.data); }
template<unsigned __int8 index_0, unsigned __int8 index_1, unsigned __int8 index_2, unsigned __int8 index_3> __forceinline sse4Floats shuffle( const sse4Floats &input ) { return shuffle<index_0, index_1, index_2, index_3>(input.data); }
__forceinline sse4Floats expand( const sse4Floats &input, const unsigned __int8 index ) { return expand(input.data, index); }
__forceinline sse4Floats expand( const float input ) { return vectorize(input); }
__forceinline sse1Float hadd( const sse4Floats &input ) { return hadd(input.data); }
