#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "ssebasic.h"
#include "ssemask.h"



/*
 * Warren Hunt : 2006-10-09
 * A wrapper for a 4-vector of ints implemented in sse
 *
 */
struct sse4Ints {
	__m128 data;
	__forceinline sse4Ints( const __m128 input = sseZeros() )
		: data(input) {}
	__forceinline sse4Ints( const __int32 input )
		: data(_my_set1_epi32(input)) {}
	__forceinline sse4Ints( const __int32 input_0, const __int32 input_1, const __int32 input_2, const __int32 input_3 )
		: data(_my_set_epi32(input_3, input_2, input_1, input_0)) {}

	__forceinline __int32  operator []( const unsigned __int32 index ) const { assume(index < 4); return ((__int32 *)&data)[index]; }
	__forceinline __int32 &operator []( const unsigned __int32 index )       { assume(index < 4); return ((__int32 *)&data)[index]; }

	__forceinline sse4Ints operator +( const sse4Ints &input ) const { return _my_add_epi32(data, input.data); }
	__forceinline sse4Ints operator -( const sse4Ints &input ) const { return _my_sub_epi32(data, input.data); }

	__forceinline sse4Ints &operator +=( const sse4Ints &input ) { return *this = *this + input; }
	__forceinline sse4Ints &operator -=( const sse4Ints &input ) { return *this = *this - input; }

	__forceinline sseMask operator ==( const sse4Ints &input ) const { return _my_cmpeq_epi32(data, input.data); }
	__forceinline sseMask operator < ( const sse4Ints &input ) const { return _my_cmplt_epi32(data, input.data); }
	__forceinline sseMask operator > ( const sse4Ints &input ) const { return _my_cmpgt_epi32(data, input.data); }

	__forceinline sse4Ints operator -( void ) const { return sse4Ints() - *this; }
	__forceinline sse4Ints operator ~( void ) const { return sseNot(data); }

	__forceinline sse4Ints operator &( const sse4Ints &input ) const { return _mm_and_ps(data, input.data); }
	__forceinline sse4Ints operator |( const sse4Ints &input ) const { return _mm_or_ps(data, input.data); }
	__forceinline sse4Ints operator ^( const sse4Ints &input ) const { return _mm_xor_ps(data, input.data); }

	__forceinline sse4Ints &operator &=( const sse4Ints &input ) { return *this = *this & input; }
	__forceinline sse4Ints &operator |=( const sse4Ints &input ) { return *this = *this | input; }
	__forceinline sse4Ints &operator ^=( const sse4Ints &input ) { return *this = *this ^ input; }

	__forceinline sse4Ints operator &( const sseMask &input ) const { return _mm_and_ps(data, input.data); }
	__forceinline sse4Ints operator |( const sseMask &input ) const { return _mm_or_ps(data, input.data); }
	__forceinline sse4Ints operator ^( const sseMask &input ) const { return _mm_xor_ps(data, input.data); }

	__forceinline sse4Ints &operator &=( const sseMask &input ) { return *this = *this & input; }
	__forceinline sse4Ints &operator |=( const sseMask &input ) { return *this = *this | input; }
	__forceinline sse4Ints &operator ^=( const sseMask &input ) { return *this = *this ^ input; }
};

__forceinline sse4Ints operator &( const sseMask &input, const sse4Ints &value ) { return value & input; }
__forceinline sse4Ints operator |( const sseMask &input, const sse4Ints &value ) { return value | input; }
__forceinline sse4Ints operator ^( const sseMask &input, const sse4Ints &value ) { return value ^ input; }

__forceinline sse4Ints blend( const sseMask &mask, const sse4Ints &input_0, const sse4Ints &input_1 ) { return blend(mask.data, input_0.data, input_1.data); }
template<unsigned __int8 index_0, unsigned __int8 index_1, unsigned __int8 index_2, unsigned __int8 index_3> __forceinline sse4Ints shuffle( const sse4Ints &input ) { return shuffle<index_0, index_1, index_2, index_3>(input.data); }
__forceinline sse4Ints expand( const sse4Ints &input, const unsigned __int8 index ) { return expand(input.data, index); }
