#pragma once
/*
 * Warren Hunt : 2006-10-09
 *
 */

#include "ssebasic.h"



/*
 * Warren Hunt : 2006-10-09
 * A wrapper for a 4-vector of 32 bit masks implemented in sse
 *
 */
struct sseMask {
	__m128 data;
	__forceinline sseMask( const __m128 input = sseZeros() )
		: data(input) {}
	__forceinline sseMask( const bool input )
		: data(_my_cmpeq_epi32(_my_set1_epi32(!input), sseZeros())) {}
	__forceinline sseMask( const bool &input_0, const bool &input_1, const bool &input_2, const bool &input_3 ) 
		: data(_my_cmpeq_epi32(_my_set_epi32(!input_3, !input_2, !input_1, !input_0), sseZeros())) {}

	__forceinline sseMask operator ==( const sseMask &input ) const { return _my_cmpeq_epi32(data, input.data); }
	__forceinline sseMask operator !=( const sseMask &input ) const { return ~(*this == input); }
	__forceinline sseMask operator ~( void ) const { return *this ^ sseOnes(data); }

	__forceinline sseMask operator &( const sseMask &input ) const { return _mm_and_ps(data, input.data); }
	__forceinline sseMask operator |( const sseMask &input ) const { return _mm_or_ps (data, input.data); }
	__forceinline sseMask operator ^( const sseMask &input ) const { return _mm_xor_ps(data, input.data); }

	__forceinline sseMask &operator &=( const sseMask &input ) { return *this = *this & input; }
	__forceinline sseMask &operator |=( const sseMask &input ) { return *this = *this | input; }
	__forceinline sseMask &operator ^=( const sseMask &input ) { return *this = *this ^ input; }

	__forceinline unsigned __int32 mask( void ) const { return _mm_movemask_ps(data); }
};

__forceinline bool and4( const sseMask &input ) { return input.mask() == 0xf; }
__forceinline bool or4 ( const sseMask &input ) { return input.mask() != 0x0; }

__forceinline sseMask blend( const sseMask &mask, const sseMask &input_0, const sseMask &input_1 ) { return blend(mask.data, input_0.data, input_1.data); }
template<unsigned __int8 index_0, unsigned __int8 index_1, unsigned __int8 index_2, unsigned __int8 index_3> __forceinline sseMask shuffle( const sseMask &input ) { return shuffle<index_0, index_1, index_2, index_3>(input.data); }
__forceinline sseMask expand( const sseMask &input, const unsigned __int8 index ) { return expand(input.data, index); }

__forceinline sseMask sseFalse( void ) { return sseZeros(); }
__forceinline sseMask sseTrue( const sseMask &anything = sseFalse() ) { return sseOnes(anything.data); }
