#pragma once
/*
 * Warren Hunt : 2006-10-09
 * This file contains wrappers for basic functions having to do with __m128
 *
 */

#include "m128ifixes.h"
#include <xmmintrin.h>
#include <emmintrin.h>
#include "standard.h"



// shifting wrappers (shift instructions are immediate as of sse3 and require templating)
template<__int32 count> __forceinline __m128 shiftLeft32( const __m128 input ) { return _my_slli_epi32<count>(input); }
template<__int32 count> __forceinline __m128 shiftRightLogical32( const __m128 input ) { return _my_srli_epi32<count>(input); }
template<__int32 count> __forceinline __m128 shiftRightArithmetic32( const __m128 input ) { return _my_srai_epi32<count>(input); }
template<__int32 count> __forceinline __m128 shiftLeft128( const __m128 input ) { return _my_slli_si128<count>(input); }
template<__int32 count> __forceinline __m128 shiftRight128( const __m128 input ) { return _my_srli_si128<count>(input); }

// vector manipulation wrappers
__forceinline __m128 blend( const __m128 mask, const __m128 input_0, const __m128 input_1 ) { return _mm_or_ps(_mm_and_ps(mask, input_0), _mm_andnot_ps(mask, input_1)); }
template<unsigned __int8 index_0, unsigned __int8 index_1, unsigned __int8 index_2, unsigned __int8 index_3> __forceinline __m128 shuffle( const __m128 input ) { return _mm_shuffle_ps(input, input, _MM_SHUFFLE(index_3, index_2, index_1, index_0)); }
template<unsigned __int8 index_0, unsigned __int8 index_1, unsigned __int8 index_2, unsigned __int8 index_3> __forceinline __m128 mix( const __m128 input_0, const __m128 input_1 ) { return _mm_shuffle_ps(input_0, input_1, _MM_SHUFFLE(index_3, index_2, index_1, index_0)); }
__forceinline __m128 expand( const __m128 input, const unsigned __int8 index ) { assume(index < 4); switch ( index ) { case 0: return shuffle<0, 0, 0, 0>(input); break; case 1: return shuffle<1, 1, 1, 1>(input); break; case 2: return shuffle<2, 2, 2, 2>(input); break; case 3: return shuffle<3, 3, 3, 3>(input); break; default: assume(false); return input; } }
__forceinline __m128 vectorize( const float input ) { return _mm_set1_ps(input); }
__forceinline __m128 hadd( const __m128 input ) { return _mm_add_ps(_mm_add_ps(input, shuffle<1,2,3,0>(input)), _mm_add_ps(shuffle<2,3,0,1>(input), shuffle<3,0,1,2>(input))); }

// true, false, abs, and not
__forceinline __m128 sseZeros( void ) { return _mm_setzero_ps(); }
__forceinline __m128 sseOnes( const __m128 anything = sseZeros() ) { return _my_cmpeq_epi32(anything, anything); }
__forceinline __m128 abs( const __m128 input ) { return shiftRightLogical32<1>(shiftLeft32<1>(input)); }
__forceinline __m128 sseNot( const __m128 input ) { return _mm_xor_ps(input, sseOnes(input)); }

// load and store operations
#pragma warning(push)
#ifdef __INTEL_COMPILER
#pragma warning(disable : 1684) // warning #1684: conversion from pointer to same-sized integral type (potential portability problem)
#endif
__forceinline void store4( float *address, const __m128 input ) { assume((reinterpret_cast<size_t>(address)&0xf) == 0x0); _mm_store_ps(address, input); }
__forceinline void store4unaligned( float *address, const __m128 input ) { _mm_storeu_ps(address, input); }
__forceinline void stream4( float *address, const __m128 input ) { assume((reinterpret_cast<size_t>(address)&0xf) == 0x0); _mm_stream_ps(address, input); }
__forceinline __m128 load4( const float *address ) { assume((reinterpret_cast<size_t>(address)&0xf) == 0x0); return _mm_load_ps(address); }
__forceinline __m128 load4unaligned( const float *address ) { return _mm_loadu_ps(address); }
#pragma warning(pop)

__forceinline void store1( float *address, const __m128 input ) { _mm_store_ss(address, input); }
__forceinline __m128 load1( const float *address ) { return _mm_load_ss(address); }



// small values w/o using a load
__forceinline __m128 sse0x0( void ) { return sseZeros(); }
__forceinline __m128 sse0x1( const __m128 anything = sseZeros() ) { return shiftRightLogical32<31>(sseOnes(anything)); }
__forceinline __m128 sse0x2( const __m128 anything = sseZeros() ) { return shiftLeft32<1>(shiftRightLogical32<31>(sseOnes(anything))); }
__forceinline __m128 sse0x3( const __m128 anything = sseZeros() ) { return shiftRightLogical32<30>(sseOnes(anything)); }
__forceinline __m128 sse0x4( const __m128 anything = sseZeros() ) { return shiftLeft32<2>(shiftRightLogical32<31>(sseOnes(anything))); }
__forceinline __m128 sse0x6( const __m128 anything = sseZeros() ) { return shiftLeft32<1>(shiftRightLogical32<30>(sseOnes(anything))); }
__forceinline __m128 sse0x7( const __m128 anything = sseZeros() ) { return shiftRightLogical32<29>(sseOnes(anything)); }
__forceinline __m128 sse0x8( const __m128 anything = sseZeros() ) { return shiftLeft32<3>(shiftRightLogical32<31>(sseOnes(anything))); }
__forceinline __m128 sse0xc( const __m128 anything = sseZeros() ) { return shiftLeft32<2>(shiftRightLogical32<30>(sseOnes(anything))); }
__forceinline __m128 sse0xe( const __m128 anything = sseZeros() ) { return shiftLeft32<1>(shiftRightLogical32<29>(sseOnes(anything))); }
__forceinline __m128 sse0xf( const __m128 anything = sseZeros() ) { return shiftRightLogical32<28>(sseOnes(anything)); }
__forceinline __m128 sse0x10( const __m128 anything = sseZeros() ) { return shiftLeft32<4>(shiftRightLogical32<31>(sseOnes(anything))); }
