#pragma once
/*
 * Warren Hunt : 2006-10-09
 * General algorithms
 *
 */

#include "datastructures.h"



/*
 * Warren Hunt : 2006-10-09
 * A templated shell sort algorithm (remarkably fast)
 *
 */
template<typename Type> __forceinline void shellSort( Array<Type> &list, const Range range )
{
	// Magic shell sort numbers
	const static unsigned __int32 shellNumbers[16] = {
		1391376, 463792, 198768, 86961,
		33936, 13776, 4592, 1968, 861,
		336, 112, 48, 21, 7, 3, 1
	};

	// Shell sort (multiple insertion sort)
	for ( unsigned __int32 iteration = 0 ; iteration < 16 ; iteration++ )
	{
		const unsigned __int32 stride = shellNumbers[iteration];
		for ( unsigned __int32 index = range.start + stride ; index < range.end ; index++ )
		{
			const Type swap = list[index];
			unsigned __int32 cursor = index;
			for ( ; (cursor  >= stride) && (swap < list[cursor - stride]) ; cursor -= stride )
				list[cursor] = list[cursor - stride];
			list[cursor] = swap;
		}
	}
}
/*
 * Failed (slow) Vectorized version
 *
 */
/*{
	const unsigned __int32 size = planes.size;
	for ( unsigned __int32 iteration = 15 ; iteration < 16 ; iteration++ )
	{
		printf("!");
		const unsigned __int32 stride = 4*shellNumbers[iteration];
		for ( unsigned __int32 index = stride ; index < size ; index += 4 )
		{
			sse4Floats here = load4(&planes[index]);
			unsigned __int32 cursor = index;
			for ( ; cursor >= stride ; cursor -= stride )
			{
				const sse4Floats next = load4(&planes[cursor - stride]);
				const sseMask lessthan = here < next;
				if ( !or4(lessthan) ) break;
				store4(&planes[cursor], blend(lessthan, next, here).data);
				here = blend(lessthan, here, next);
			}
			store4(&planes[cursor], here.data);
		}
	}
}*/