#pragma once
/*
 * Warren Hunt : 2006-10-09
 * General data structures
 *
 */



/*
 * Warren Hunt : 2006-10-09
 * A pair sturcture defined outside the STL
 *
 */
template<typename Type_0, typename Type_1> struct Pair {
	Type_0 first;
	Type_1 second;

	__forceinline Pair( const Type_0 &input_first, const Type_1 &input_second )
		: first(input_first), second(input_second) {}
};

template<typename Type_0, typename Type_1> __forceinline Pair<Type_0, Type_1> makePair( const Type_0 &input_first, const Type_1 &input_second ) { return Pair<Type_0, Type_1>(input_first, input_second); }



/*
 * Warren Hunt : 2006-10-09
 * A discrete range structure [start, end)
 *
 */
struct Range {
	const unsigned __int32 start;
	const unsigned __int32 end;

	__forceinline Range( void )
		: start(0), end(0) {}
	__forceinline Range( const unsigned __int32 input_start, const unsigned __int32 input_end )
		: start(input_start), end(input_end) { assume(input_end >= input_start); }
	__forceinline Range( const Range &input )
		: start(input.start), end(input.end) {}

	__forceinline Range &operator =( const Range &input ) { new (this) Range(input); return *this; }

	__forceinline unsigned __int32 length( void ) const { return end - start; }
};

// a macro to loop over a range
#define forRange(range, name) for ( unsigned __int32 (name) = (range).start ; (name) < (range).end ; (name)++ )



/*
 * Warren Hunt : 2006-10-09
 * A templated array defined outside the STL
 *
 */
template<typename Type> class Array {
	Type *data;

public:
	const unsigned __int32 size;

	__forceinline Array( void )
		: size(0), data(0x0) {}
	__forceinline Array( const unsigned __int32 input_size )
		: size(input_size), data((Type *)_aligned_malloc(input_size*sizeof(Type), 16)) { assume(size > 0); }
	__forceinline Array( const Array &input )
		: size(input.size), data(0x0)
	{
		if ( input.size > 0 )
			data = (Type *)_aligned_malloc(input.size*sizeof(Type), 16);
		for ( unsigned __int32 i = 0 ; i < input.size ; i++ )
			new (&data[i]) Type(input[i]);
	}
	__forceinline ~Array( void ) { if ( data != 0x0 ) _aligned_free(data); data = 0x0; }

	__forceinline Type &operator []( const unsigned __int32 index ) const { assume(index < size); return data[index]; }
	__forceinline Type &operator []( const unsigned __int32 index )       { assume(index < size); return data[index]; }

	__forceinline Range range( void ) const { return Range(0, size); }
	__forceinline Array &resize( const unsigned __int32 input_size )
	{
		Type *temp = data;
		const unsigned __int32 copyLimit = (size < input_size) ? size : input_size;
		this->Array(input_size);
		for ( unsigned __int32 i = 0 ; i < copyLimit ; i++ )
			new (&data[i]) Type(temp[i]);
		if ( temp != 0x0 )
			_aligned_free(temp);
		return *this;
	}

	__forceinline Array operator =( const Array &input ) { ~Array(); new (this) Array(input); }
};
